% Modified by David Torun for International Macroeconomics (MEcon, FS2021)
%tot_mx_var_cbc.m
%produces country-specific and country medians of the theoretical (MX model)  and empirical (SVAR model)  shares of variances explained by terms of trade shocks of TB, Y, C, and I. Tables 7.6 and 7.7  in  the chapter entitled ``Importable Goods, Exportable Goods, and the Terms of Trade,'' (chapter 7) 
%of the book  ``Open Economy Macroeconomics,'' by Mart�n Uribe and Stephanie Schmitt Grohe, 2014.
%� Mart�n Uribe and Stephanie Schmitt-Groh�, 2014. 

clear all; close all; fclose('all'); clc;

seed = 1234; %for reproducibility
rng(seed)

tic;
%read empirical  share of variacnes explained by tot shocks
load 'tot_cbcs_own5.mat' ncou nv var_svar var_svar_tot var_svar_tot_ratio_cbc country country_iso hx PI
% % tot_cbcs_own5.mat is produced by detrend_tot_cbcs_Q8_01.m 
% % --> gives us estimates of hx and PI

RHO_TOT_cbc = hx(1,1,:);
STD_TOT_cbc = PI(1,1,:);
clear hx PI

% % Countries to keep (ROW vector of ISO 3 Codes):
keep_cou = [ "CAN" "CHE" "FIN"];

% create country indices to keep only countries specified in "keep_cou"
ind = zeros(1,size(country,2));

for i = 1:size(country,2)
    for j = 1:numel(keep_cou)
        if isequal(country_iso{i},keep_cou{j})
           ind(1,i) = 1;
        end
    end
end
clear keep_cou i j;

ncou = sum(ind);
country = country(ind==1);
country_iso = country_iso(ind==1);
var_svar = var_svar(:,ind==1);
var_svar_tot = var_svar_tot(:,ind==1);
var_svar_tot_ratio_cbc = var_svar_tot_ratio_cbc(:,ind==1);

RHO_TOT_cbc = RHO_TOT_cbc(1,1,ind==1);
STD_TOT_cbc = STD_TOT_cbc(1,1,ind==1);

clear ind;

% % % Model
mx_model %analytic solution

% %theoretical predictions from MX  model
var_model_tot = zeros(nv,ncou);

% % % Calibration
runs = 10; %how many different starting values do you want to try?

for k=1:ncou %for each country

disp(' ')    
fprintf('country: ' ) 
fprintf(country{k})    
disp(' ')    
    
RHO_TOT = RHO_TOT_cbc(k);
STD_TOT = STD_TOT_cbc(k);    
    
    % set starting values for calibration! 
    % -> similar to reasoning in Section 7.6.6
    % -> however, we simply set PHIX=PHIM !!!
    PHI_start = 1;
    PSSI_start = 1;

    objective=@(X) obj_to_min_Q8(X,var_svar_tot(:,k),RHO_TOT,STD_TOT); %target second moments 
        %function obj_to_min_Q8(X) computes SS values for given parameter
        %values, and then verifies whether the specified parameters roughly
        %match the second moments we target (see code obj_to_min_Q8.m --> there
        %the metric is specified! --> if the match is not good enough (where
        %good is defined by the "FunctionTolerance" criterion below), it
        %computes a further iteration

        X_start=[PHI_start,PSSI_start]; %starting values
            %inputs are, in this order, PHI (=PHIM=PHIX) and PSSI
            %importantly, parameters should be strictly positive!
        
        lb=[0.00001, 0.00001]; %lower bound values for the six parameters 
        ub=[30,30]; %upper bound values for the six parameters
        % -> same boundaries as in the notes of Table 7.4!

        options = optimoptions(@fmincon,'FunctionTolerance', 1e-8, 'MaxIter', 100, 'StepTolerance', 1e-8,...
                        'MaxFunctionEvaluations',500,'ConstraintTolerance',1e-8,'Display','off'); 
            %specify tolerance value; default is 1e-6
            
        for iter=1:runs
            
            disp(' ')
            disp(strcat(['Running iteration nr. ' num2str(iter) ' of ' num2str(runs) ' for ' char(country{k})]))

            % Add random change to starting values
            if iter~=1 %first iteration with initial starting values
                random_change = rand(size(X_start)).*3; 
                X_start = X_start.*random_change;

                    %check that all values satisfy the bounds
                    for i = 1:length(X_start)
                        if X_start(i)<lb(i) 
                            X_start(i)=lb(i);
                        elseif X_start(i)>ub(i)
                            X_start(i)=ub(i);
                        end
                    end
            end
            
                disp(strcat(['Starting values: PHI = ' num2str(X_start(1)) ' and PSSI = ' num2str(X_start(2))]))

            try 
            %we use try - catch - end in order to avoid being stuck in a
            %situation where we cannot find an equilibrium!
            %--> i.e., when we do NOT find an equilibrium with the
            %specified starting values X_start, then we set the parameter
            %values to NaN for that iteration
                [PARAMETERS,fval,exitflag,output] = fmincon(objective,X_start,[],[],[],[],lb,ub,[],options); %optimization

            catch ME
                disp(strcat(['Warning: Starting values ' num2str(X_start) ' yield no Equilibrium']))
                PARAMETERS = nan(size(X_start));
                continue
            end

        [PARAMETERS,fval,exitflag,output] = fmincon(objective,X_start,[],[],[],[],lb,ub,[],options); %optimization

            %keep PARAMETERS that minimize objective retrieved using
            %different starting values
            if iter==1
                PARAMETERS_opt = PARAMETERS;
            elseif sum(isnan(PARAMETERS))==0 && sum(isnan(PARAMETERS_opt))==0  
                if objective(PARAMETERS)<objective(PARAMETERS_opt)
                    PARAMETERS_opt = PARAMETERS;
                end
            elseif sum(isnan(PARAMETERS_opt))>0
                PARAMETERS_opt = PARAMETERS;
            end
        
        end
        
PARAMETERS=PARAMETERS_opt;
clear PARAMETERS_opt;
%extract optimal parameter values
PHI = PARAMETERS(1);
PHIX = PHI;
PHIM = PHI;
PSSI=PARAMETERS(2);

COUNTRY_PARAMETERS(:,k) = PARAMETERS;

%recompute SS with optimal parameter values
[sigy0,sigx0,nivv,noutput,ntot,ntby,nc] = mx_function_Q8(PHIM,PHIX,PSSI,RHO_TOT,STD_TOT);

        %%%Check matching:
        data_var(:,k) = [sqrt(var_svar_tot(5,k)/var_svar_tot(3,k))  ;  %sigma_i / sigma_y
        sqrt(var_svar_tot(2,k)/var_svar_tot(3,k))]; %sigma_tb / sigma_y

        % model-implied sigma_i / sigma_y --> similar to Table 7.4
        model_var(1,k) = sqrt(sigy0(nivv,nivv)/sigy0(noutput,noutput));
        % model-implied sigma_tb / sigma_y --> similar to Table 7.4
        model_var(2,k) = sqrt(sigy0(ntby,ntby)/sigy0(noutput,noutput));

%         sum((model_var(1,k)-data_var(1,k)).^2)+sum((model_var(2,k)-data_var(2,k)).^2)

diag(sigy0);  %unconditional variances conditional on tot shocks
var_model_tot(2:end,k) = ans([ntby;noutput;nc;nivv]); %pick variables of interest (output, consumption, investment, trade balance divided by trend output)

var_model_tot(1,k) = sigx0(ntot,ntot);

end

var_model_tot_ratio_cbc = var_model_tot./ var_svar;

var_svar_tot_ratio = median(var_svar_tot_ratio_cbc,2);

var_model_tot_ratio = median(var_model_tot_ratio_cbc,2);

var_table = [var_model_tot_ratio var_svar_tot_ratio]*100;

Table_var_7_6 = array2table([(var_model_tot_ratio*100) (var_svar_tot_ratio*100)] , ...
    'VariableNames',{'MX_Model', 'Empirical_SVAR_Model'});

Table_var_7_6

xxx = zeros(2*nv,ncou);
xxx(1:2:2*nv,:) =var_model_tot_ratio_cbc*100;
xxx(2:2:2*nv,:)  = var_svar_tot_ratio_cbc*100;

country{ncou+1} = "Median";

xxx_new = xxx;
for i = 1:size(xxx,1)
    xxx_new(i,ncou+1) = median(xxx(i,:));
end

xxx = xxx_new; 
clear xxx_new;

Table_var_7_7 = array2table([table2array(cell2table(country))'  num2str(xxx(1,:)') num2str(xxx(2,:)') ...
    num2str(xxx(3,:)') num2str(xxx(4,:)') num2str(xxx(5,:)') num2str(xxx(6,:)') ...
    num2str(xxx(7,:)') num2str(xxx(8,:)') num2str(xxx(9,:)') num2str(xxx(10,:)')] , ...
    'VariableNames',{'Country_name', ...
    'tot_MX', 'tot_SVAR', ...
    'tb_MX', 'tb_SVAR', ...
    'y_MX', 'y_SVAR', ...
    'c_MX', 'c_SVAR', ...
    'i_MX', 'i_SVAR'});

Table_var_7_7

COUNTRY_PARAMETERS_new = COUNTRY_PARAMETERS;
for i = 1:size(COUNTRY_PARAMETERS,1)
    COUNTRY_PARAMETERS_new(i,ncou+1) = median(COUNTRY_PARAMETERS(i,:));
end

COUNTRY_PARAMETERS = COUNTRY_PARAMETERS_new; 
clear COUNTRY_PARAMETERS_new;

data_var_new = data_var;
for i = 1:size(data_var,1)
    data_var_new(i,ncou+1) = median(data_var(i,:));
end

data_var = data_var_new; 
clear data_var_new;

model_var_new = model_var;
for i = 1:size(model_var,1)
    model_var_new(i,ncou+1) = median(model_var(i,:));
end

model_var = model_var_new; 
clear model_var_new;

% PHIM=PHIX
COUNTRY_PARAMETERS(3,:) = COUNTRY_PARAMETERS(2,:);
COUNTRY_PARAMETERS(2,:) = COUNTRY_PARAMETERS(1,:);

Table_7_4 = array2table([table2array(cell2table(country))'  num2str(COUNTRY_PARAMETERS(1,:)')  ...
    num2str(COUNTRY_PARAMETERS(2,:)') num2str(COUNTRY_PARAMETERS(3,:)') ...
    num2str(data_var(1,:)') num2str(model_var(1,:)') num2str(data_var(2,:)') num2str(model_var(2,:)') ] , ...
    'VariableNames',{'Country_name', ...
    'PHIM', 'PHIX', 'PSI', ...
    'sigma_i_to_sigma_y_data','sigma_i_to_sigma_y_model', ...
    'sigma_tb_to_sigma_y_data','sigma_tb_to_sigma_y_model'});

Table_7_4

PHIM_cbc(1,:) = COUNTRY_PARAMETERS(1,1:end-1); % take out median
PHIX_cbc(1,:) = COUNTRY_PARAMETERS(2,1:end-1); % take out median
PSSI_cbc(1,:) = COUNTRY_PARAMETERS(3,1:end-1); % take out median

save parameters.mat PHIM_cbc PHIX_cbc PSSI_cbc RHO_TOT_cbc STD_TOT_cbc country_iso

time_used = toc;
time_used